/*
  Emul.c  Crude SQ device emulator
        
	Usage:  Emul [-C1] [-B57600] [-P0]\n");
	        -C1    Sets the COM port # to use\n");
	        -B57600 Sets the bps to use\n");
	        -P25    Sets the delay after each char in ms.\n");
	              (Values shown above are the defaults)\n");
    
	  The program produces a continuous series of 5 packets:

	Roll = 0.1, Pitch= 0  With a checksum error
	Roll =   0, Pitch= 0  Checksum correct
	Roll =  10, Pitch=10  Checksum correct
	Roll =  20, Pitch=20  Checksum correct
	Roll =  30, Pitch=30  Checksum correct

    Repeated (regardless of any commands received)
	
	  To use, connect the output of this program to another
	serial port using a null modem cable. The other port
	will appear in many respects (not all) as an SQ inclinometer
	device.

  History:
  11/11/04 pmt.
*****************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <io.h>
#include <conio.h>
#include "serial.h"


#define DEBUG 0


//****************************************************************************
void main(int argc, char *argv[])
{
int argN;
unsigned int checksum, count;
int Pause = 25;					// Millisecs to pause between packets
	// validate arguments which might over-ride the .ini values (??)
	if (argc < 2)   	
	{
		fprintf (stderr,"\nUsage:  Emul [-C1] [-B57600] [-P0]\n");
		fprintf (stderr,"        -C1      Sets the COM port # to use\n");
		fprintf (stderr,"        -B115200 Sets the bps to use\n");
		fprintf (stderr,"        -P25     Sets the delay after each packet in ms.\n\n");
		fprintf (stderr,"                 Values given above are the default arguments.\n");
		fprintf (stderr,"                 At least one argument MUST be given\n");
		
		exit(1);
	}


	/*
		Set default values
	*/

	for(argN = 1; argN < argc; ++argN)
	{

		if ((argN > 1) && (argv[argN][0] != '-')) 	// No -, should be a file name..
		{
			fprintf(stderr, "\nArguments begin with '-'! unknown: %s", argv[argN]);
		}

		
		switch(argv[argN][1])  // Switch on char following '-'
		{
			case 'b':		// Baud rate ??
			case 'B':
					COM_baud = atoi(&argv[argN][2]);
					if ((COM_baud <2400) || (COM_baud > 120000)) 
					{
						printf("Baud rate = %d is invalid!\n", COM_baud);
						return;
					}
			  break;


			case 'c':		// Com port
			case 'C': 
					COM_port = atoi(&argv[argN][2]);
					if ((COM_port <1) || (COM_port > 255)) {
						printf("COM port COM%d invalid!\n", COM_port);
						return;
					}
			  break;


			case 'p':
			case 'P':
			  Pause = atoi(&argv[argN][2]);
			  printf("Will pause for %d millisecs between Packets sent.\n", Pause);
			  break;
		}
	}


	//******************************************************

	printf("Emul: Opening COM%d at %d bps.\n", COM_port, COM_baud);
	printf("      Pausing %d ms between packets\n", Pause);
	
	COM_init();			// Set up the serial port for Galil commands

	printf("Sending packets.  Hit a key to stop.\n");
	count = 0;

	while (1)                           // for each packet
    {
		
		if ((++count %100) == 0) printf("%d\r", count);

		checksum = 0;
		
		if (!COMputchar((char)0xFE))			// Send Preamble to serial port
		{
			CloseCOM();
			exit(1);
		}
		COMputchar((char)0xFE);				// Send Preamble to serial port
		
		// Send a bad packet
		COMputchar(1);						// Invalidate the 0 checksum for 1 pkt

		for (int i=0; i<7; i++)				// All zeros for rest of the packet
		{
			COMputchar(0); 
		}
		Sleep(Pause);						// Packet spacing

		// Now 4 good packets (0,10,20,30 pitch and roll)
		for (int j=0; j<4; j++)
		{
			checksum = 0;
		
			COMputchar((char)0xFE);			// Send Preamble to serial port
			COMputchar((char)0xFE);			// Send Preamble to serial port
			
			for (int i=0; i<3; i++)			// 3 pairs: pitch,roll,undefined
			{
				COMputchar(0);				// High byte of Pitch/Roll
				COMputchar(j);				// Low byte: Pitch*10
				checksum += j;
			}

			// Now output Checksum (assuming <256 !)
			COMputchar(0);					// High byte 
			COMputchar(checksum);			// Low byte
		
			Sleep(Pause);					// Packet spacing
		}
		
		if(_kbhit())
		{
			CloseCOM();						// Free up the COM port
			printf(" User interrupted sending! Sleeping 5 second\n");
			Sleep(5000);

			break;
		}
	} // while 1

} // End Main()

